#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_DISKINODES

/*
config KNOERRE_CHECK_DISKINODES
    bool "check_diskinodes"
    default y
    help
      This knoerre module shows inode usage in % like "df -i" does.
      More information offers man-page knoerre(1).

config KNOERRE_CHECK_DISKINODES_KEY
    string "key string for check_diskinodes"
    default "diskinodes/"
    depends on KNOERRE_CHECK_DISKINODES
    help
      The key string for check_diskinodes as described in man-page knoerre(1).
*/

/**
 * This function gets disk usage stats and writes it as http data out.
 *
 * This function is called by knoerre main().
 * It gets disk usage stats for a given mount point or file,
 * writes it as http data out and exists.
 * $Id: do_check_diskinodes.c 641 2013-05-11 22:55:29Z fwbergm $
 *
 * @param buf The http data buf for returning data.
 * @param buflen The size of the http data buf.
 * @param bufp The pointer to char after http-header in buf.
 * @param arg1 The file/directory to analyze.
 */
int do_check_diskinodes(register char *buf, const int buflen, char *bufp, char *arg1) {
  register char *p;
  register uint64_t a;
  register uint64_t s;
  struct statfs stat_buf;

  p = strchr(arg1, ' ');
  if (!p) p = strchr(arg1, '\n');
  *p = 0;

  if (statfs(arg1, &stat_buf) != 0) goto error;

  p = stpcpy(bufp, arg1);
  *p++ = ' ';
  a = stat_buf.f_ffree;
  p = sputlongs(p, a);
  p = stpcpy(p-1, " inodes free");
  *p++ = '\n';

  /* percentage */
  s = stat_buf.f_files - a; /* used inodes */
  a = stat_buf.f_files;
  if (a == 0) {
    /* reiserfs, btrfs, fat */
    s = 0;
  } else {
    if ((a + 50) < a) {
      /* extremely rare case... need cpu's carry bit ;-) */
      a /= 100;
    } else {
      a = (a + 50) / 100;
    }
    if (a != 0) s /= a;
  }

  p = sputlongs(p, s);
#ifdef KNOERRED
  return p - buf;
#endif
  write(1, buf, p-buf);
  _exit(0);

  error:
#ifdef KNOERRED
  return 0;
#endif
  check_error_exit(0);
  return 0;

}
#endif
